/**
 * Adapts a {@code Counter} to serve as a {@code CallbackB}.
 *
 * @author Tom Verhoeff (Eindhoven University of Technology)
 */
class CounterAdapter implements CallbackB {
    
    /** Counter being adapted. */
    private final Counter counter;
    
    /**
     * Constructs a new adapter for a given counter
     * 
     * @param counter  counter to adapt
     * @pre {@code counter != null}
     * @throws IllegalArgumentException  if {@code counter == null}
     */
    public CounterAdapter(Counter counter) {
        if (counter == null) {
            throw new IllegalArgumentException(
                    "CounterAdapter: pre counter != null failed");
        }
        this.counter = counter;
    }
    
    /**
     * Gets the counter being adapted.
     * 
     * @return counter being adapted
     */
    public Counter getCounter() {
        return counter;
    }
    
    /**
     * Counts a data item.  The content of the item is ignored.
     * 
     * @param data  data item to count
     */
    @Override
    public void doB(String data) {
        counter.count();
    }
    
}
