/**
 * Filter to selectively pass data to another {@code CallbackB}.
 *
 * @author Tom Verhoeff (Eindhoven University of Technology)
 */
class DataFilter implements CallbackB {
    
    /** Callback to invoke if data matches the filter condition */
    private CallbackB destination;
    
    /** Pattern to select */
    private String pattern;
    
    /**
     * Constructs a new data filter with given condition.
     * 
     * @param destination  the callback to pass on to
     * @param pattern  the pattern to match
     * @pre {@code destination != null}
     * @throws IllegalArgumentException  if {@code destination == null}
     */
    public DataFilter(CallbackB destination, String pattern) {
        if (destination == null) {
            throw new IllegalArgumentException(
                    "DataQuoter: pre destination != null failed");
        }
        this.destination = destination;
        this.pattern = pattern;
    }
    
    /**
     * Filters the data item, that is, passes it on when
     * it contains the given pattern.
     * 
     * @param data  data item to filter
     */
    @Override
    public void doB(String data) {
        if (data.contains(pattern)) {
            destination.doB(data);
        }
    }
    
}
