/**
 * Put quotes around data and pass on to another {@code CallbackB}.
 *
 * @author Tom Verhoeff (Eindhoven University of Technology)
 */
class DataQuoter implements CallbackB {
    
    /** Callback to pass data on to */
    private CallbackB destination;
    
    /**
     * Constructs a new data quoter with given destination.
     * 
     * @param destination  the callback to pass on to
     * @pre {@code destination != null}
     * @throws IllegalArgumentException  if {@code destination == null}
     */
    public DataQuoter(CallbackB destination) {
        if (destination == null) {
            throw new IllegalArgumentException(
                    "DataQuoter: pre destination != null failed");
        }
        this.destination = destination;
    }
    
    /**
     * Passes on the quoted data item.
     * 
     * @param data  data item to quote
     */
    @Override
    public void doB(String data) {
        destination.doB('"' + data + '"');
    }
    
}
