/**
 * Main environment, using callbacks to couple functionalities A and B.
 *
 * @author Tom Verhoeff (Eindhoven University of Technology)
 */
public class Main {

    /**
     * Invokes functionality A with various processors for functionality B.
     * 
     * @pre {@code args.length >= 2}
     */
    public static void main(String[] args) {
        assert args.length >= 3 : "main(args): args.length >= 3 failed";
        char experiment = args[0].charAt(0);
        final int n = Integer.parseInt(args[1]);
        switch (experiment) {
        
        // Experiment (a): Stateless callback
        case 'a': {
        assert args.length >= 3 : "main(args): args.length >= 3 failed";
        final CallbackB printer = new DataPrinter(args[2]);
        FunctionalityA.doA(n, printer);
        break; }
        
        // Experiment (b): Anonymous stateless callback
        case 'b': {
        FunctionalityA.doA(n, new CallbackB() {
            int count;
            @Override
            public void doB(String data) {
                ++ count;
                System.out.println("Item " + count + " counted");
            }
        });
        break; }
            
        // Experiment (c): Callback with state, adapter using inheritance
        case 'c': {
        final DataCounter counter = new DataCounter();
        FunctionalityA.doA(n, counter);
        System.out.println("Counter received " + counter.getCount()
                + " items from A");
        break; }
        
        // Experiment (d): Callback with state, adapter using composition
        case 'd': {
        final Counter counter = new DataCounter();
        final CounterAdapter adapter = new CounterAdapter(counter);
        FunctionalityA.doA(n, adapter);
        System.out.println("Counter received " + counter.getCount()
                + " items from A");
        break; }
        
        // Experiment (e): Composite callback
        case 'e': {
        assert args.length >= 3 : "main(args): args.length >= 3 failed";
        final CallbackB printer = new DataPrinter(args[2]);
        final DataCounter counter = new DataCounter();
        final CompositeCallbackB printer_counter
                = new CompositeCallbackB();
        printer_counter.add(printer);
        printer_counter.add(counter);
        FunctionalityA.doA(n, printer_counter);
        System.out.println("Counter received " + counter.getCount()
                + " items from A");
        break; }
        
        // Experiment (f): Decorator callback
        case 'f': {
        final CallbackB printer = new DataPrinter(args[2]);
        final CallbackB quoter = new DataQuoter(printer);
        FunctionalityA.doA(n, quoter);
        break; }
        
        // Experiment (g): Decorator callback that filters
        case 'g': {
        final DataCounter counter = new DataCounter();
        final CallbackB filter = new DataFilter(counter, args[2]);
        FunctionalityA.doA(n, filter);
        System.out.println("Counter received " + counter.getCount()
                + " items from A after filtering");
        break; }
        
        }
    }
    
}